Class: ProcessExecuter::ProcessIOError
- Inherits:
-
Error
- Object
- Error
- ProcessExecuter::ProcessIOError
- Defined in:
- lib/process_executer/errors.rb
Overview
Raised if an exception occurred while processing subprocess output
Raised if an exception occurred while processing subprocess output