Class: ProcessExecuter::SignaledError
- Inherits:
-
CommandError
- Object
- CommandError
- ProcessExecuter::SignaledError
- Defined in:
- lib/process_executer/errors.rb
Overview
Raised when the command exits because of an uncaught signal
Raised when the command exits because of an uncaught signal