Method: ProcessExecuter::DestinationBase.handles?
- Defined in:
- lib/process_executer/destination_base.rb
.handles?(destination) ⇒ Boolean
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
Determines if this class can handle the given destination
This is an abstract class method that must be implemented by subclasses.
62 63 64 |
# File 'lib/process_executer/destination_base.rb', line 62 def self.handles?(destination) raise NotImplementedError end |